//
// Vertex shader for drawing drawing a toy ball
//
// Author: OGLSL implementation by Someone
//
// Copied directly from the OrangeBook
//

varying vec4 ECposition;    // surface position in eye coords
varying vec4 ECballCenter;  // ball center in eye coords

uniform vec4 LightDir;
uniform vec4 HVector;
uniform vec4 SpecularColor;
uniform vec4 Red, Yellow, Blue;

uniform vec4 HalfSpace0;
uniform vec4 HalfSpace1;
uniform vec4 HalfSpace2;
uniform vec4 HalfSpace3;
uniform vec4 HalfSpace4;

uniform float InOrOutInit;
uniform float StripeWidth;
uniform float FWidth;

void main(void)
{
    vec4 normal;
    vec4 p;
    vec4 surfColor;
    float intensity;
    vec4 distance;
    float inorout;

    p.xyz = normalize(ECposition.xyz - ECballCenter.xyz);
    p.w = 1.0;

    inorout = InOrOutInit;

    distance[0] = dot(p, HalfSpace0);
    distance[1] = dot(p, HalfSpace1);
    distance[2] = dot(p, HalfSpace2);
    distance[3] = dot(p, HalfSpace3);

    distance += 0.2;
    distance = smoothstep(-FWidth, FWidth, distance);

    inorout += dot(distance, vec4(1.0));

    distance.x = dot(p, HalfSpace4) + 0.2;
    distance.y = StripeWidth - abs(p.z);
    distance = smoothstep(-FWidth, FWidth, distance);
    inorout += distance.x;

    inorout = clamp(inorout, 0.0, 1.0);

    surfColor = mix(Yellow, Red, inorout);
    surfColor = mix(surfColor, Blue, distance.y);

    normal = p;

    intensity = 0.2;
    intensity += 0.8 * clamp(dot(LightDir, normal), 0.0, 1.0);
    surfColor *= intensity;

    intensity = clamp(dot(HVector, normal), 0.0, 1.0);
    intensity = pow(intensity, SpecularColor.a);

    surfColor += SpecularColor * intensity;
    gl_FragColor = vec4(surfColor, 1.0);
}